/*
 * Copyright (c) 1996, 2002 Sandro Sigala.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef GAMEWIDGET_H
#define GAMEWIDGET_H

#include <fx.h>

#include "fxrepton.h"
#include "gameengine.h"

class GameWidget: public FXCanvas {
	FXDECLARE(GameWidget)

private:
      	FXBMPImage *sprites[num_sprites];
      	FXBMPImage *masks[num_masks];
      	FXBMPImage *pausebmp;
        FXBMPImage *dbuf;
	int last_width, last_height;
	int gwidth, gheight;
	int scr_dwidth, scr_dheight;
        int xb, yb;
	int zoom;
	int anim_status;

	void computeGameSize();
	void drawBoard();

protected:
	GameWidget();

private:
  	GameWidget(const GameWidget&);
  	GameWidget& operator =(const GameWidget&);

public:
      	long onPaint(FXObject *, FXSelector, void *);

public:
     	enum {
        	ID_REPAINT = FXCanvas::ID_LAST,
		ID_GAME,
            	ID_LAST
    	};

public:
	GameEngine engine;
	Animation anim;

      	GameWidget(FXComposite *p, FXObject *tgt, FXSelector sel = 0, FXuint opts = FRAME_NORMAL,
       		   FXint x = 0, FXint y = 0, FXint w = 0, FXint h = 0);
	virtual ~GameWidget();

	virtual void create();

        virtual FXint getDefaultWidth();
        virtual FXint getDefaultHeight();

	void playSound(Sound snd);
	void showAnimation(Animation a);
       	void loadLevel(const FXString& episodepath, const FXString& spritespath, int levelno, bool resetscore);
       	void loadSprites(const FXString& spritespath);
	int getZoomValue() const { return zoom; }
	void setZoomValue(int value);
};

#endif // !GAMEWIDGET_H

