/*
 * Copyright (c) 1996, 2002 Sandro Sigala.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "legendwindow.h"

const struct {
	SpriteObj sprite;
	int group;
	const char *desc;
} descs[] = {
	{ Obj_Repton,      1, "Repton." },
	{ Obj_Empty,	   2, "An empty space." },
	{ Obj_Ground1,     3, 0 },
	{ Obj_Ground2,     3, "Ground.\nRepton can dig into this." },
	{ Obj_Wall,        4, 0 },
	{ Obj_Filled_Wall, 4, 0 },
	{ Obj_Wall_Mirror, 4, "Walls." },
        { Obj_Wall_North_West, 5, 0 },
	{ Obj_Wall_North_East, 5, 0 },
	{ Obj_Wall_South_West, 5, 0 },
	{ Obj_Wall_South_East, 5, "Walls." },
	{ Obj_Filled_Wall_North_West, 6, 0 },
	{ Obj_Filled_Wall_North_East, 6, 0 },
	{ Obj_Filled_Wall_South_West, 6, 0 },
	{ Obj_Filled_Wall_South_East, 6, "Walls." },
	{ Obj_Rock,	   7, "A falling object.\nRepton can move this." },
	{ Obj_Bomb,        8, "Get this to finish the level." },
	{ Obj_TimeCapsule, 9, "Time capsule.  Resets the countdown." },
	{ Obj_Transport,   10, "A transporter." },
	{ Obj_Diamond,     11, "You must collect all those objects\nto finish a level." },
 	{ Obj_Crown,       12, "Increments your score." },
	{ Obj_Key,         13, "A key.\nOpens the strongboxes." },
	{ Obj_Strongbox,   14, "A strongbox.\nWhen you get a key, it turns itself into a diamond." },
	{ Obj_Monster,     15, 0 },
        { Obj_Egg,         15, "A monster and its egg." },
	{ Obj_Spirit,      16, 0 },
	{ Obj_Cage,        16, "A monster and a cage.\nWill turn into a diamond if it finds a cage." },
	{ Obj_Fungus,      17, "Do not touch this.\nAfter some time, can replicate itself." },
	{ Obj_Skull,       18, "Do not touch this, or you will die." },
};
const int size_descs = sizeof descs / sizeof descs[0];

FXIMPLEMENT(LegendWindow, FXTopWindow, 0, 0)

LegendWindow::LegendWindow()
{
}

LegendWindow::LegendWindow(FXApp *a, const FXString& name, const FXString& episode,
			   FXIcon *icon, FXIcon *miniicon)
    : FXTopWindow(a, name, icon, miniicon, DECOR_CLOSE|DECOR_TITLE|DECOR_BORDER, 0,0,0,0, 0,0,0,0, 0,0)
{
	for (int i = 0; i < num_sprites; ++i) {
		sprites[i] = new FXBMPImage(getApp(), 0, IMAGE_SHMI|IMAGE_SHMP,
					    sprite_width, sprite_height);
		sprites[i]->create();
	}

	loadSprites(episode);

	FXVerticalFrame *contents =
        	new FXVerticalFrame(this, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 5,5,5,5, 5,5);
       	FXHorizontalFrame *h0 =
        	new FXHorizontalFrame(contents, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 			0,0,0,0, 0,0,0,0, 5,5);
	FXVerticalFrame *v1 =
        	new FXVerticalFrame(h0, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 0,0,0,0, 0,0);
      	new FXVerticalSeparator(h0, SEPARATOR_GROOVE|LAYOUT_FILL_Y);
	FXVerticalFrame *v2 =
        	new FXVerticalFrame(h0, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 		0,0,0,0, 0,0,0,0, 0,0);

	FXHorizontalFrame *h = 0;
	int lastgroup = -1;
	const int vsize = 9;
	int y = 0;
	num_views = 0;
	for (int i = 0; i < size_descs; ++i) {
  		if (!h || descs[i].group != lastgroup) {
			++y;
       			h = new FXHorizontalFrame(y > vsize ? v2 : v1, LAYOUT_SIDE_LEFT|LAYOUT_FILL_X|LAYOUT_FILL_Y,
       	 			0,0,0,0, 5,5,5,5, 5,5);
    		}
  		FXHorizontalFrame *mapframe =
   			new FXHorizontalFrame(h, FRAME_THICK|FRAME_SUNKEN|LAYOUT_FILL_Y,
   				0,0,0,0, 0,0,0,0, 0,0);
		views[num_views++] = new FXImageView(mapframe, sprites[descs[i].sprite], 0, 0, LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT,
					0,0,sprite_width,sprite_height);
		if (descs[i].desc)
		    	new FXLabel(h, descs[i].desc, 0, JUSTIFY_LEFT|LAYOUT_CENTER_Y|LAYOUT_FILL_X);
		lastgroup = descs[i].group;
	}
}

LegendWindow::~LegendWindow()
{
	for (int i = 0; i < num_sprites; ++i)
		delete sprites[i];
}

void LegendWindow::create()
{
  	FXTopWindow::create();
}

void LegendWindow::loadSprites(const FXString& episode)
{
	FXBMPImage *img = new FXBMPImage(getApp(), 0);

	FXString filename = episode + PATHSEPSTRING "sprites.bmp";

	// Load it
  	FXFileStream stream;
  	if(!stream.open(filename, FXStreamLoad)) {
    		FXMessageBox::error(this, MBOX_OK, "Error", "Cannot load %s", filename.text());
  		exit(1);
     	}

	img->loadPixels(stream);
	stream.close();
	img->create();

	if (img->getWidth() != 8*sprite_width || img->getHeight() != 6*sprite_height) {
    		FXMessageBox::error(this, MBOX_OK, "Error", "The sprite bitmap %s must be 256x192", filename.text());
  		exit(1);
     	}

	// Load the sprites
	for (int y = 0; y < 6; ++y)
		for (int x = 0; x < 8; ++x) {
			int i = y*8 + x;
			FXDCWindow dc(sprites[i]);
		   	dc.setFunction(BLT_SRC);
			dc.drawImage(img, -x*sprite_width, -y*sprite_height);
		}

 	delete img;
}

void LegendWindow::updateSprites(const FXString& episode)
{
	loadSprites(episode);
	for (int i = 0; i < num_views; ++i)
		views[i]->update();
}

